@@ -15,8 +15,8 @@ from PIL import Image, ExifTags
 # Create your models here.
 
 
-ASSIGNMENT_IMG_SIZE = (1024,768)
-COURSE_IMG_SIZE = (800,600)
+ASSIGNMENT_IMG_SIZE = (1024, 768)
+COURSE_IMG_SIZE = (800, 600)
 
 
 # 压缩上传作业图片
@@ -81,6 +81,7 @@ class Course(models.Model):
     teacher = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, verbose_name='授课老师')
     image = models.ImageField(null=True, blank=True, verbose_name='配图', upload_to='course/face')
     status = models.IntegerField(verbose_name='状态', default=0)  # 0 不显示，1 显示在列表
+    template = models.CharField(max_length=180, null=True, blank=True, verbose_name='模版')
 
     class Meta:
         db_table = 'bee_django_course_course'
@@ -160,8 +161,6 @@ class Section(models.Model):
         return question
 
 
-
-
 # 课件的用户笔记
 class UserSectionNote(models.Model):
     section = models.ForeignKey(Section)
@@ -219,6 +218,7 @@ class Video(models.Model):
     file_name = models.CharField(max_length=180, null=True, blank=True, verbose_name='视频文件名')
     info = models.TextField(verbose_name='说明', null=True, blank=True)
     created_at = models.DateTimeField(auto_now_add=True)
+
     # created_by = models.ForeignKey("users.User", related_name='cc_video_user', null=True)  # 由谁上传
     # video_status = models.IntegerField(default=1, null=True)  # 状态 1-正常，2-正常（删除视频）
 
@@ -412,7 +412,6 @@ class UserLive(models.Model):
             return False
 
 
-
 class UserLiveComment(models.Model):
     user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name='作者')
     user_live = models.ForeignKey(UserLive)
@@ -448,20 +447,27 @@ class UserCourse(models.Model):
         self.save()
 
 
-
 class UserCourseSection(models.Model):
     user_course = models.ForeignKey(UserCourse)
     section = models.ForeignKey(Section)
     created_at = models.DateTimeField(auto_now_add=True)
-    updated_at = models.DateTimeField(default=timezone.now)
+    learned_at = models.DateTimeField(verbose_name='开始学习时间', blank=True, null=True)
+    passed_at = models.DateTimeField(verbose_name='课件通过时间', blank=True, null=True)
     status = models.IntegerField(default=0)  # 0 未开始， 1 学习中，2 通过，3 退回重修, 4 提交
     score = models.IntegerField(blank=True, null=True, verbose_name='得分')
     work_time = models.IntegerField(verbose_name='总共练习时间',
                                     null=True, blank=True, default=0)  # 学生总共练习时间
+    minus_live_mins = models.IntegerField(verbose_name='扣减的时间', default=0)
+    teacher_add_mins = models.IntegerField(verbose_name='助教增加的练习时间', null=True, blank=True)
+    teacher_add_at = models.DateTimeField(verbose_name='助教操作时间', null=True, blank=True)
+    send_message_at = models.DateTimeField(verbose_name='给助教发消息时间', null=True, blank=True)
     comment = models.TextField(null=True, blank=True, verbose_name='评语')
     question_passed = models.BooleanField(default=False)
     question_passed_at = models.DateTimeField(null=True, blank=True)
 
+    # 准备弃用
+    updated_at = models.DateTimeField(default=timezone.now)
+
     class Meta:
         db_table = 'bee_django_course_user_course_section'
         ordering = ['-created_at']
@@ -473,8 +479,9 @@ class UserCourseSection(models.Model):
             ('pass_ucs', '能通过学生课件'),
             ('close_ucs', '能关闭学生课件'),
             ('open_ucs', '能开启学生课件'),
+            ('minus_live_mins', '能扣减学生直播时长'),
             ('review_ucs', '能查看学生作业'),
-            ('score_ucs', '能给学生作业评分')
+            ('score_ucs', '能给学生作业评分'),
         ]
 
     def close(self):
@@ -506,7 +513,7 @@ class UserCourseSection(models.Model):
             imagework_check = True
 
         if section.has_videowork:
-            if self.work_time >= section.video_length_req:
+            if self.work_time - self.minus_live_mins >= section.video_length_req:
                 videowork_check = True
         else:
             videowork_check = True
@@ -537,6 +544,7 @@ class UserCourseSection(models.Model):
             self.get_pass()
         else:
             self.status = 1
+            self.learned_at = timezone.now()
             self.updated_at = timezone.now()
             self.save()
 
@@ -570,6 +578,7 @@ class UserCourseSection(models.Model):
 
         self.score = score
         self.comment = comment
+        self.passed_at = timezone.now()
         self.updated_at = timezone.now()
         self.status = 2
         self.save()
@@ -606,6 +615,18 @@ class UserCourseSection(models.Model):
         if rc:
             self.get_pass()
 
+    # 如果有直播需要，则计算时间倍数，给各个项目独立使用
+    # 1倍：发消息 2倍：发邮件
+    def get_time_multiple(self):
+        if not self.section.has_videowork:
+            return 0
+        work_time = self.work_time - self.minus_live_mins
+        req_time = self.section.video_length_req
+        if req_time > 0:
+            return work_time / req_time
+        else:
+            return 0
+
     def next_section(self, is_open=False):
         order_sets = CourseSectionMid.objects.filter(course=self.user_course.course,
                                                      section=self.section)
@@ -643,11 +664,10 @@ class UserCourseSection(models.Model):
         except:
             return 0
 
-
     # 获取学生已通过的课件
     @classmethod
-    def get_user_pass_sections(cls,user_course):
-        return cls.objects.filter(user_course=user_course,status=2)
+    def get_user_pass_sections(cls, user_course):
+        return cls.objects.filter(user_course=user_course, status=2)
 
 
 class UserAssignment(models.Model):
@@ -693,41 +713,41 @@ def resize_assignment_image_handler(sender, **kwargs):
 class Preference(models.Model):
     how_to_pass = models.IntegerField(verbose_name='课程通过模式', default=0)  # 0 自动， 1 手动
 
-
-# 提醒助教和客服的默认处理方式
-from bee_django_message.exports import send_message
-from .exports import get_current_user_course, get_user_last_course_section
-
-
-def notify_mentor(self):
-    user_class = self.userprofile.user_class
-    if not user_class:
-        return -1
-    if not user_class.assistant:
-        return -2
-
-    mentor = self.userprofile.user_class.assistant
-    ucs = get_user_last_course_section(self)
-    title = self.userprofile.name() + u'学习的课程[' + ucs.section.name + u']已经达标'
-
-    uc = get_current_user_course(self)
-    url = '/course/user_course_detail/' + str(uc.id)
-
-    send_message(to_user=mentor, title=title, url=url)
-
-    return 0
-
-def notify_agent(self):
-    uc = get_current_user_course(self)
-    ucs = get_user_last_course_section(self)
-    url = '/course/user_course_detail/' + str(uc.id)
-
-    title = self.userprofile.name() + u'学习的课程[' + ucs.section.name + u']已经达标'
-
-    send_message(from_user=self, title=title, url=url, message_identity='course_notify')
-
-    return 0
-
-
-User.add_to_class('notify_mentor', notify_mentor)
-User.add_to_class('notify_agent', notify_agent)
+# # 提醒助教和客服的默认处理方式
+# from bee_django_message.exports import send_message
+# from .exports import get_current_user_course, get_user_last_course_section
+#
+#
+# def notify_mentor(self):
+#     user_class = self.userprofile.user_class
+#     if not user_class:
+#         return -1
+#     if not user_class.assistant:
+#         return -2
+#
+#     mentor = self.userprofile.user_class.assistant
+#     ucs = get_user_last_course_section(self)
+#     title = self.userprofile.name() + u'学习的课程[' + ucs.section.name + u']已经达标'
+#
+#     uc = get_current_user_course(self)
+#     url = '/course/user_course_detail/' + str(uc.id)
+#
+#     send_message(to_user=mentor, title=title, url=url)
+#
+#     return 0
+#
+#
+# def notify_agent(self):
+#     uc = get_current_user_course(self)
+#     ucs = get_user_last_course_section(self)
+#     url = '/course/user_course_detail/' + str(uc.id)
+#
+#     title = self.userprofile.name() + u'学习的课程[' + ucs.section.name + u']已经达标'
+#
+#     send_message(from_user=self, title=title, url=url, message_identity='course_notify')
+#
+#     return 0
+#
+#
+# User.add_to_class('notify_mentor', notify_mentor)
+# User.add_to_class('notify_agent', notify_agent)
