@@ -7,6 +7,8 @@ assignment_will_be_scored = Signal(providing_args=["user_course_section", "reque
 # 作业评分后发出的信号
 assignment_was_scored = Signal(providing_args=["user_course_section", "request"])
 
+# 更新课件直播时长后发送的信号
+ucs_update_time = Signal(providing_args=['user_course_section'])
 # 课程通过后发送的信号
 ucs_passed = Signal(providing_args=['user_course_section'])
 
