@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-import json, os, random,datetime
+import json, os, random, datetime
 from django.forms.models import model_to_dict
 from django.shortcuts import render
 from dss.Serializer import serializer
@@ -21,9 +21,9 @@ from .models import Course, Section, CourseSectionMid, Video, UserLive, UserCour
 from .forms import CourseForm, SectionForm, CourseSectionForm, CourseSectionOrderForm, VideoForm, UploadImageForm, \
     UserCourseForm, UserAssignmentForm, UserAssignmentImageForm, SectionVideoForm, PreferenceForm, \
     section_attach_form, UserSectionNoteForm, SectionVideoOrderForm, LivePlayLogsForm, UserLiveCommentForm, \
-    CourseSectionQuestionSearchForm
-from .utils import JSONResponse, page_it,can_view_expired_live
-from .cc import create_video_info,get_live_str
+    CourseSectionQuestionSearchForm, UserLiveSearchForm
+from .utils import JSONResponse, page_it, can_view_expired_live
+from .cc import create_video_info, get_live_str
 from .cc import get_video_str as get_cc_video_str
 from .qiniu_api import get_video_str as get_qiniu_video_str
 from django.http import JsonResponse, HttpResponseRedirect
@@ -390,11 +390,11 @@ def prepare_live_video_page_data(live_video_id):
 
 def live_video_detail(request, user_live_id):
     user_live = UserLive.objects.get(pk=user_live_id)
-    if user_live.provider_name=='cc':
+    if user_live.provider_name == 'cc':
         live_url = get_live_str(user_live.record_video_id)
     else:
-        live_url=None
-    can_view=can_view_expired_live(request.user,user_live)
+        live_url = None
+    can_view = can_view_expired_live(request.user, user_live)
     comments = user_live.userlivecomment_set.order_by('-submit_date')
 
     if request.method == "POST":
@@ -410,9 +410,9 @@ def live_video_detail(request, user_live_id):
         comment_form = UserLiveCommentForm()
 
     return render(request, 'bee_django_course/live/live_video_detail.html', context={
-        "can_view":can_view,
+        "can_view": can_view,
         'user_live': user_live,
-        'live_url':live_url,
+        'live_url': live_url,
         'comments': comments,
         'comment_form': comment_form,
     })
@@ -540,22 +540,19 @@ def cc_video_callback(request):
     return
 
 
-
-
 def uptoken(request):
     key = request.GET.get('key')
     token = get_qiniu_token(key)
     if hasattr(settings, "QINIU_CONFIG"):
-        domain=settings.QINIU_CONFIG.domain
+        domain = settings.QINIU_CONFIG.domain
     else:
-        domain=None
+        domain = None
     return JsonResponse(data={
         'uptoken': token,
         'domain': domain,
     })
 
 
-
 # 查看所有的学生录播
 @method_decorator(cls_decorator(cls_name='LiveList'), name='dispatch')
 class LiveList(ListView):
@@ -589,20 +586,75 @@ class LiveList(ListView):
         else:
             q3 = Q(pk=None)
 
-        self.queryset = UserLive.objects.filter(q1|q2|q3).distinct()
+        self.queryset = UserLive.objects.filter(q1 | q2 | q3).distinct()
         user = self.get_user()
         if user:
             self.queryset = self.queryset.filter(user=user)
+        user_name = self.request.GET.get("user_name")
+        provider = self.request.GET.get("provider")
+        if not provider in [0, "0", None]:
+            self.queryset = self.queryset.filter(provider_name=provider)
+        if not provider in [0, "0", None]:
+            try:
+                kwargs = {}  # 动态查询的字段
+                name_field = settings.USER_NAME_FIELD
+                kwargs["user__" + name_field + '__icontains'] = user_name
+                self.queryset = self.queryset.filter(**kwargs)
+            except:
+                pass
         return self.queryset
 
     def get_context_data(self, **kwargs):
         context = super(LiveList, self).get_context_data(**kwargs)
         # context["vdieo_provider_name"] = settings.COURSE_VIDEO_PROVIDER_NAME
         context["live_url"] = None
-        context["user"] = self.get_user()
+        user = self.get_user()
+        context["user"] = user
+        user_name = self.request.GET.get("user_name")
+        provider = self.request.GET.get("provider")
+        # 全部
+        if not user:
+            if hasattr(settings, "COURSE_LIVE_PROVIDER_LIST"):
+                provider_list = (("0", '全部'),)
+                for _provider in settings.COURSE_LIVE_PROVIDER_LIST:
+                    provider_list += ((_provider, _provider),)
+            else:
+                provider_list = None
+            context["search_form"] = UserLiveSearchForm(provider_list, {"user_name": user_name, "provider": provider})
         return context
 
 
+class CustomUserLivePage(ListView):
+    template_name = 'bee_django_course/custom_user/custom_user_live.html'
+    context_object_name = 'live_list'
+    paginate_by = 20
+    queryset = None
+
+
+    def _get_user(self):
+        user = User.objects.get(id=self.kwargs["user_id"])
+        return user
+
+    def get_context_data(self, **kwargs):
+        context = super(CustomUserLivePage, self).get_context_data(**kwargs)
+        user=self._get_user()
+        if hasattr(settings, "CC_CONFIG") and 'cc' in settings.COURSE_LIVE_PROVIDER_LIST:
+            cc_userid = settings.CC_CONFIG.user_id
+        else:
+            cc_userid = None
+        if hasattr(settings, "GENSEE_CONFIG"):
+            from bee_django_course.gensee import get_room_info
+            gensee_room = get_room_info(user.userprofile.gensee_room_id)
+        else:
+            gensee_room = None
+        context['cc_userid']=cc_userid
+        context['gensee_room']=gensee_room
+        return context
+
+    def get_queryset(self):
+        return UserLive.objects.filter(user_id=self.kwargs["user_id"], status=1).order_by('-start_time')
+
+
 # # 查看单个学生的所有录播
 # @method_decorator(cls_decorator(cls_name='UserLiveList'), name='dispatch')
 # class UserLiveList(ListView):
@@ -616,7 +668,7 @@ class LiveList(ListView):
 #
 #     def get_context_data(self, **kwargs):
 #         context = super(UserLiveList, self).get_context_data(**kwargs)
-#         # TODO
+#
 #         user = get_object_or_404(User, pk=self.kwargs["user_id"])
 #         # context["vdieo_provider_name"] = settings.COURSE_VIDEO_PROVIDER_NAME
 #         context["live_url"] = None
@@ -702,49 +754,74 @@ def user_course(request, user_id):
     })
 
 
+# =========用户课程课件===========
 # 查看指定用户user的课件列表
-class UserCourseSectionList(ListView):
-    template_name = "bee_django_course/user/user_course_section_list.html"
-    model = UserCourseSection
-    queryset = None
-    paginate_by = 30
-    context_object_name = 'user_course_section_list'
-    user_course = None
-    user_course_list = []
+class UserCourseSectionRedirectView(RedirectView):
+    query_string = True
 
-    def get_queryset(self):
+    def _get_url(self, user_course):
+        return reverse('bee_django_course:user_course_detail', kwargs={"user_course_id": user_course.id})
+
+
+    def _get_no_course_url(self):
+        return reverse('bee_django_course:user_course_detail',kwargs={"user_course_id":0})
+
+
+    def get_redirect_url(self, *args, **kwargs):
         user_id = self.kwargs["user_id"]
-        self.user_course_list = UserCourse.objects.filter(user__id=user_id).order_by('status', '-created_at')
+        user_course_list = UserCourse.objects.filter(user__id=user_id).order_by('status', '-created_at')
+        if user_course_list.exists():
+            user_course = user_course_list.first()
+            self.url = self._get_url(user_course)
+        else:
+            self.url = self._get_no_course_url()
+        return super(UserCourseSectionRedirectView, self).get_redirect_url(*args, **kwargs)
 
-        if self.user_course_list.exists():
-            self.user_course = self.user_course_list.first()
-            self.section_list = get_user_course_section_list(self.user_course)
-            return self.section_list
-        return []
+    # template_name = "bee_django_course/user/user_course_section_list.html"
+    # model = UserCourseSection
+    # queryset = None
+    # paginate_by = 30
+    # context_object_name = 'user_course_section_list'
+    # user_course = None
+    # user_course_list = []
+    #
+    # def get_queryset(self):
+    #     user_id = self.kwargs["user_id"]
+    #     self.user_course_list = UserCourse.objects.filter(user__id=user_id).order_by('status', '-created_at')
+    #
+    #     if self.user_course_list.exists():
+    #         self.user_course = self.user_course_list.first()
+    #         self.section_list = get_user_course_section_list(self.user_course)
+    #         return self.section_list
+    #     return []
+    #
+    # def get_context_data(self, **kwargs):
+    #     context = super(UserCourseSectionList, self).get_context_data(**kwargs)
+    #     context["user_course"] = self.user_course
+    #     context["user_course_list"] = self.user_course_list
+    #     return context
 
-    def get_context_data(self, **kwargs):
-        context = super(UserCourseSectionList, self).get_context_data(**kwargs)
-        context["user_course"] = self.user_course
-        context["user_course_list"] = self.user_course_list
-        return context
+
+class CostomUserCourseSectionRedirectView(UserCourseSectionRedirectView):
+    def _get_url(self, user_course):
+        return reverse('bee_django_course:custom_user_course_detail', kwargs={"user_course_id": user_course.id})
 
 
 # 用户查看指定课程的课件列表
 # 弃用 改用UserCourseDetail
-def user_course_section_list(request, user_course_id):
-    user_course = get_object_or_404(UserCourse, pk=user_course_id)
-
-    section_list = get_user_course_section_list(user_course=user_course)
-
-    return render(request, 'bee_django_course/user/user_course_section_list.html', context={
-        'user_course': user_course,
-        'section_list': section_list,
-    })
+# def user_course_section_list(request, user_course_id):
+#     user_course = get_object_or_404(UserCourse, pk=user_course_id)
+#
+#     section_list = get_user_course_section_list(user_course=user_course)
+#
+#     return render(request, 'bee_django_course/user/user_course_section_list.html', context={
+#         'user_course': user_course,
+#         'section_list': section_list,
+#     })
 
 
 # 查看指定用户课程user_course的课件列表
 class UserCourseDetail(ListView):
-    template_name = "bee_django_course/user/user_course_section_list.html"
     model = UserCourseSection
     queryset = None
     paginate_by = 30
@@ -752,9 +829,19 @@ class UserCourseDetail(ListView):
     user_course = None
     user_course_list = []
 
+    def get_template_names(self):
+        if self.user_course:
+            return "bee_django_course/user/user_course_section_list.html"
+        else:
+            return "bee_django_course/course/no_course.html"
+
+
     def get_queryset(self):
         user_course_id = self.kwargs["user_course_id"]
-        self.user_course = UserCourse.objects.get(id=user_course_id)
+        try:
+            self.user_course = UserCourse.objects.get(id=user_course_id)
+        except:
+            return []
         user = self.user_course.user
         self.user_course_list = UserCourse.objects.filter(user=user).order_by('status', '-created_at')
         self.section_list = get_user_course_section_list(self.user_course)
@@ -767,6 +854,28 @@ class UserCourseDetail(ListView):
         return context
 
 
+class CustomUserCourseDetail(UserCourseDetail):
+    def get_template_names(self):
+        if self.user_course:
+            course = self.user_course.course
+        else:
+            return 'bee_django_course/custom_user/no_course.html'
+
+        if course.template in [0, "0", None]:
+            return 'bee_django_course/custom_user/custom_user_course_section_list.html'
+        else:
+            # 特殊模版，各项目外重写html
+            return 'bee_django_course/custom_user/custom_user_course_section_list' + course.template + '.html'
+
+
+class CustomUserCourseList(ListView):
+    template_name = 'bee_django_course/custom_user/custom_user_course_list.html'
+    model = UserCourse
+    paginate_by = 30
+    context_object_name = 'user_course_list'
+
+
+
 # 用户课程列表的管理页面
 def manage_user_course_section_list(request, user_course_id):
     user_course = get_object_or_404(UserCourse, pk=user_course_id)
@@ -854,6 +963,9 @@ class UserCourseSectionDetail(DetailView):
             return redirect(self.get_success_url())
 
 
+class CustomUserCourseSectionDetail(UserCourseSectionDetail):
+    template_name = 'bee_django_course/custom_user/custom_user_course_section_detail.html'
+
 # 用户查看作业
 def user_assignment(request, ucs_id):
     user_course_section = get_object_or_404(UserCourseSection, pk=ucs_id)
@@ -983,12 +1095,12 @@ def user_assignment_image_upload(request, ucs_id):
 
 
 # 用户查看课程
-def view_courses(request):
-    courses = Course.objects.order_by('created_at')
-
-    return render(request, 'bee_django_course/user/view_courses.html', context={
-        'courses': courses,
-    })
+# def view_courses(request):
+#     courses = Course.objects.order_by('created_at')
+#
+#     return render(request, 'bee_django_course/user/view_courses.html', context={
+#         'courses': courses,
+#     })
 
 
 @transaction.atomic
@@ -1073,10 +1185,9 @@ def manage_user_assignments(request):
     else:
         q3 = Q(pk=None)
 
-
-    data_list = UserCourseSection.objects.filter(status__in=[1, 4])\
-        .filter(Q(userassignment__isnull=False) | Q(userassignmentimage__isnull=False))\
-        .filter(q1|q2|q3).distinct().order_by('-updated_at')
+    data_list = UserCourseSection.objects.filter(status__in=[1, 4]) \
+        .filter(Q(userassignment__isnull=False) | Q(userassignmentimage__isnull=False)) \
+        .filter(q1 | q2 | q3).distinct().order_by('-updated_at')
 
     if student_name:
         data_list = data_list.filter(Q(user_course__user__first_name__contains=student_name)
@@ -1192,52 +1303,60 @@ def close_user_course_section(request, ucs_id):
 
 
 # 手动通过时，用户提醒助教
-def notify_mentor(request, user_id):
+def notify_mentor(request, ucs_id):
     try:
-        user = User.objects.get(pk=user_id)
-        rc = user.notify_mentor()
-
-        if rc == 0:
-            return JsonResponse(data={
-                'rc': 0,
-                'msg': '提醒成功'
-            })
-        elif rc == -1:
+        ucs = UserCourseSection.objects.get(pk=ucs_id)
+        user = ucs.user_course.user
+        uc = ucs.user_course
+        user_class = user.userprofile.user_class
+        if not user_class:
             return JsonResponse(data={
                 'rc': -1,
                 'msg': '用户还没有班级'
             })
-        elif rc == -2:
+        if not user_class.assistant:
             return JsonResponse(data={
                 'rc': -1,
                 'msg': '用户班级未分配助教'
             })
-        else:
-            return JsonResponse(data={
-                'rc': -1,
-                'msg': '未知错误'
-            })
-    except User.DoesNotExist:
-        return JsonResponse(data={
-            'rc': -1,
-            'msg': '用户不存在'
-        })
 
+        mentor = user.get_assistant()
+        title = user.get_user_name() + u'学习的课程[' + ucs.section.name + u']已经达标'
 
-# 手动通过时，用户申请客服
-def notify_agent(request, user_id):
-    try:
-        user = User.objects.get(pk=user_id)
-        user.notify_agent()
+        url = '/course/user_course_detail/' + str(uc.id)
 
+        from bee_django_message.models import SendRecord
+        SendRecord.send_message(to_user=mentor, title=title, url=url)
         return JsonResponse(data={
             'rc': 0,
             'msg': '提醒成功'
         })
-    except User.DoesNotExist:
+
+    except:
         return JsonResponse(data={
             'rc': -1,
-            'msg': '用户不存在'
+            'msg': '错误'
+        })
+
+
+# 手动通过时，用户申请客服
+def notify_agent(request, ucs_id):
+    try:
+        ucs = UserCourseSection.objects.get(pk=ucs_id)
+        user = ucs.user_course.user
+        uc = ucs.user_course
+        url = '/course/user_course_detail/' + str(uc.id)
+        title = user.get_user_name() + u'学习的课程[' + ucs.section.name + u']已经达标'
+        from bee_django_message.models import SendRecord
+        SendRecord.send_message(from_user=user, message_identity='course_notify', title=title, url=url)
+        return JsonResponse(data={
+            'rc': 0,
+            'msg': '提醒成功'
+        })
+    except:
+        return JsonResponse(data={
+            'rc': -1,
+            'msg': '错误'
         })
 
 
@@ -1393,7 +1512,7 @@ def cc_live_end_callback(request):
 #             date = form.cleaned_data['date']
 #             total, play_log = get_play_logs(video_id=video_id, date=date)
 #             print(total, play_log)
-            # return redirect(reverse_lazy('bee_django_crm:preuser_application_update_preuser', kwargs={'pk': preuser.id}))
+# return redirect(reverse_lazy('bee_django_crm:preuser_application_update_preuser', kwargs={'pk': preuser.id}))
 
 
 def save_coin_multiple(userlive):
@@ -1799,33 +1918,53 @@ def delete_user_live(request, user_live_id):
         })
 
 
+def save_ucs_minus_live_mins(request):
+    ucs_id = request.POST.get("ucs_id")
+    mins = request.POST.get("mins")
+    print(ucs_id, mins)
+    try:
+        ucs = UserCourseSection.objects.get(id=ucs_id)
+    except:
+        return JsonResponse(data={
+            'error': 1,
+            'msg': '参数错误'
+        })
+
+    ucs.minus_live_mins = mins
+    ucs.save()
+    return JsonResponse(data={
+        'error': 0,
+        'msg': '成功'
+    })
+
+
 # 微信小程序接口
 # 课程页
 def weixin_user_course_index(request):
     user = request.user
     if not user:
         return JsonResponse(data={
-            "errCode":1,
+            "errCode": 1,
             'msg': '无效的用户',
         })
 
     ucs = get_user_last_course_section(user)
     if not ucs:
         return JsonResponse(data={
-            "errCode":0,
+            "errCode": 0,
             'msg': '没有学习中的课程',
-            "ucs_detail":None,
-            "course_detail":None,
-            "section_detail":None
+            "ucs_detail": None,
+            "course_detail": None,
+            "section_detail": None
         })
 
     return JsonResponse(data={
-        "errCode":0,
+        "errCode": 0,
         'user_name': user.first_name or user.username,
         'section_detail': serializer(ucs.section, output_type='json',
                                      datetime_format='string', exclude_attr=['image']),
-        "course_detail":serializer(ucs.user_course.course,output_type='json',
-                                     datetime_format='string', exclude_attr=['teacher_id','image']),
+        "course_detail": serializer(ucs.user_course.course, output_type='json',
+                                    datetime_format='string', exclude_attr=['teacher_id', 'image']),
         'ucs_detail': serializer(ucs, output_type='json', datetime_format='string'),
         'msg': 'OK',
     })
@@ -1898,22 +2037,25 @@ def weixin_user_live_list(request, user_id):
         'user_live_list': serializer(user_live_list, output_type='json', datetime_format='string')
     })
 
+
 # 小程序 用户录播详情
-def weixin_user_live_detail(request,live_id):
-    user_live=UserLive.objects.get(id=live_id)
+def weixin_user_live_detail(request, live_id):
+    user_live = UserLive.objects.get(id=live_id)
     return JsonResponse(data={
         'errCode': 0,
         'user_live_detail': serializer(user_live, output_type='json', datetime_format='string')
     })
 
+
 def test_video(request):
     return render(request, 'bee_django_course/test_video.html')
 
+
 # 直播转码完成后的一系列操作：更新课件进度，发送信号
-def live_finish(userlive,timedelta=None):
+def live_finish(userlive, timedelta=None):
     if not timedelta:
-        timedelta=userlive.duration
-    if timedelta<60:
+        timedelta = userlive.duration
+    if timedelta < 60:
         return
 
     if not userlive.user:
@@ -1923,6 +2065,7 @@ def live_finish(userlive,timedelta=None):
     user_course_section = get_user_last_course_section(userlive.user)
     if user_course_section:
         user_course_section.update_work_time(timedelta / 60)
+        signals.ucs_update_time.send(sender=UserCourseSection, ucs=user_course_section)
     coin_multiple = save_coin_multiple(userlive)
     signals.live_callback_finished.send(sender=UserLive, user_live=userlive, coin_multiple=coin_multiple)
     return
@@ -1931,26 +2074,28 @@ def live_finish(userlive,timedelta=None):
 @csrf_exempt
 def tencent_live_callback(request):
     # {"appid":1256935119,"channel_id":"aa","duration":3,"end_time":1562658284,"end_time_usec":453420,"event_type":100,"file_format":"mp4","file_id":"5285890791305792056","file_size":420382,"media_start_time":3801,"record_bps":0,"record_file_id":"5285890791305792056","start_time":1562658283,"start_time_usec":375858,"stream_id":"aa","stream_param":"txSecret=e939142722088dc59849bb82fc4d0fd1&txTime=5D24B9FF","task_id":"18288319047887901972","video_id":"1256935119_55156dda6162403abd8860da89d07ab6","video_url":"http://1256935119.vod2.myqcloud.com/8f802c2cvodcq1256935119/b3d83b505285890791305792056/f0.mp4"}
-    print("tencent_live_callback: ")
+    # print("tencent_live_callback: ")
 
-    _dict=json.loads(request.body)
+    _dict = json.loads(request.body)
     try:
-        username=_dict["stream_id"]
-        video_url=_dict["video_url"]
-        start_time=int(_dict["start_time"])
-        end_time=int(_dict["end_time"])
-        duration=_dict["duration"]
+        username = _dict["stream_id"]
+        video_url = _dict["video_url"]
+        start_time = int(_dict["start_time"])
+        end_time = int(_dict["end_time"])
+        duration = _dict["duration"]
+        record_file_id = _dict["record_file_id"]
 
-        user=User.objects.get(username=username)
-        userlive=UserLive()
-        userlive.provider_name="tencent"
-        userlive.replay_url=video_url
-        userlive.start_time=datetime.datetime.fromtimestamp(start_time)
-        userlive.end_time=datetime.datetime.fromtimestamp(end_time)
-        userlive.user=user
-        userlive.duration=duration
-        userlive.status=1
-        userlive.record_status=10
+        user = User.objects.get(username=username)
+        userlive = UserLive()
+        userlive.provider_name = "tencent"
+        userlive.replay_url = video_url
+        userlive.start_time = datetime.datetime.fromtimestamp(start_time)
+        userlive.end_time = datetime.datetime.fromtimestamp(end_time)
+        userlive.record_video_id = record_file_id
+        userlive.user = user
+        userlive.duration = duration
+        userlive.status = 1
+        userlive.record_status = 10
         userlive.save()
         live_finish(userlive)
         return JsonResponse({
